@@ -34,12 +34,14 @@
  */
 
 #include <openssl/x509.h>
+#include <openssl/x509v3.h>
 #include <openssl/md5.h>
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 #include <openssl/pem.h>
 #include <openssl/rand.h>
 
+#include <assert.h>
 #include <strings.h>
 #include <string.h>
 #include <syslog.h>
@@ -78,7 +80,7 @@ init_cert_file(SSL_CTX *ctx, const char *path)
 	return (0);
 }
 
-int
+static int
 verify_server_fingerprint(const X509 *cert)
 {
 	unsigned char fingerprint[EVP_MAX_MD_SIZE] = {0};
@@ -102,7 +104,7 @@ verify_server_fingerprint(const X509 *cert)
 }
 
 int
-smtp_init_crypto(int fd, int feature, struct smtp_features* features)
+smtp_init_crypto(int fd, int feature, struct smtp_features* features, const char *server_hostname)
 {
 	SSL_CTX *ctx = NULL;
 #if (OPENSSL_VERSION_NUMBER >= 0x00909000L)
@@ -112,6 +114,8 @@ smtp_init_crypto(int fd, int feature, struct smtp_features* features)
 #endif
 	X509 *cert;
 	int error;
+	ASN1_OCTET_STRING *ip = NULL;
+	int verify_cert = 0;
 
 	/* XXX clean up on error/close */
 	/* Init SSL library */
@@ -140,11 +144,32 @@ smtp_init_crypto(int fd, int feature, struct smtp_features* features)
 		}
 	}
 
+	verify_cert = config.features & VERIFYCERT;
+	if (verify_cert) {
+		int (*verify_cb)(int, X509_STORE_CTX *) = NULL;
+		int mode = SSL_VERIFY_PEER | SSL_VERIFY_FAIL_IF_NO_PEER_CERT;
+		/* NB: Don't change verify_cb */
+                verify_cb = SSL_CTX_get_verify_callback(ctx);
+		SSL_CTX_set_verify(ctx, mode, verify_cb);
+
+		error = SSL_CTX_set_default_verify_paths(ctx);
+		if (error == 0) {
+			syslog(LOG_NOTICE, "remote delivery deferred: SSL failed to set default CA path: %s",
+			       ssl_errstr());
+			return (1);
+		}
+	}
+
+	/*
+	 * If we are in smtp_init_crypto() that means SECURETRANSFER
+	 * must be set.
+	 */
+	assert((feature & SECURETRANSFER) != 0);
+
 	/*
 	 * If the user wants STARTTLS, we have to send EHLO here
 	 */
-	if (((feature & SECURETRANS) != 0) &&
-	     (feature & STARTTLS) != 0) {
+	if ((feature & STARTTLS) != 0) {
 		/* TLS init phase, disable SSL_write */
 		config.features |= NOSSL;
 
@@ -159,6 +184,10 @@ smtp_init_crypto(int fd, int feature, struct smtp_features* features)
 					return (0);
 				}
 			}
+		} else {
+			syslog(LOG_ERR, "remote delivery deferred: could not perform server greeting: %s",
+				neterr);
+			return (1);
 		}
 
 		/* End of TLS init phase, enable SSL_write/read */
@@ -172,6 +201,51 @@ smtp_init_crypto(int fd, int feature, struct smtp_features* features)
 		return (1);
 	}
 
+	/* Only do SNI if hostname is not an IP address */
+	ip = a2i_IPADDRESS(server_hostname);
+	if (ip != NULL) {
+		ASN1_OCTET_STRING_free(ip);
+		ip = NULL;
+	} else {
+		ERR_clear_error();
+
+		error = SSL_set_tlsext_host_name(config.ssl, server_hostname);
+		if (error == 0) {
+			syslog(LOG_NOTICE, "remote delivery deferred: SSL set TLS host name failed: %s",
+			       ssl_errstr());
+			return (1);
+		}
+	}
+
+	if (verify_cert) {
+		X509_VERIFY_PARAM *param = SSL_get0_param(config.ssl);
+
+		ip = a2i_IPADDRESS(server_hostname);
+		if (ip == NULL) {
+			ERR_clear_error();
+
+			error = X509_VERIFY_PARAM_set1_host(param, server_hostname,
+							    strlen(server_hostname));
+			if (error == 0) {
+				syslog(LOG_NOTICE, "remote delivery deferred: SSL set hostname check failed: %s",
+				       ssl_errstr());
+				return (1);
+			}
+		} else {
+			error = X509_VERIFY_PARAM_set1_ip(param, ASN1_STRING_get0_data(ip),
+							  ASN1_STRING_length(ip));
+
+			ASN1_OCTET_STRING_free(ip);
+			ip = NULL;
+
+			if (error == 0) {
+				syslog(LOG_NOTICE, "remote delivery deferred: SSL set ip check failed: %s",
+				       ssl_errstr());
+				return (1);
+			}
+		}
+	}
+
 	/* Set ssl to work in client mode */
 	SSL_set_connect_state(config.ssl);
 
