@@ -99,30 +99,57 @@ static char *
 set_from(struct queue *queue, const char *osender)
 {
 	const char *addr;
+	const char *from_user = NULL;
+	const char *from_host = NULL;
 	char *sender;
+	int len_user = 0;	/* length of user */
 
-	if (config.masquerade_user) {
-		addr = config.masquerade_user;
-	} else if (osender) {
+	/* Get any sender address from parameters or environment. */
+	if (osender) {
 		addr = osender;
-	} else if (getenv("EMAIL") != NULL) {
-		addr = getenv("EMAIL");
 	} else {
-		addr = username;
+		addr = getenv("EMAIL");
 	}
 
-	if (!strchr(addr, '@')) {
-		const char *from_host = hostname();
+	if (addr) {
+		/*
+		 * Try and split into user and host.  User is not
+		 * null-terminated if the split is successful.
+		 */
+		from_user = addr;
+		from_host = strchr(addr, '@');
+		if (from_host) {
+			len_user = from_host - addr;
+			from_host++;
+		} else {
+			len_user = strlen(from_user);
+		}
+	}
 
-		if (config.masquerade_host)
-			from_host = config.masquerade_host;
+	/* Replace empty user name and/or host name with defaults. */
+	if (len_user == 0) {
+		from_user = username;
+	}
+	if (from_host == NULL || strlen(from_host) == 0) {
+		from_host = hostname();
+	}
 
-		if (asprintf(&sender, "%s@%s", addr, from_host) <= 0)
-			return (NULL);
-	} else {
-		sender = strdup(addr);
-		if (sender == NULL)
-			return (NULL);
+	/* Deal with masquerades. */
+	if (config.masquerade_user) {
+		from_user = config.masquerade_user;
+	}
+	if (config.masquerade_host) {
+		from_host = config.masquerade_host;
+	}
+
+	if (from_user != addr) {
+		/* Fix up length of user. */
+		len_user = strlen(from_user);
+	}
+
+	/* Construct final sender address. */
+	if (asprintf(&sender, "%.*s@%s", len_user, from_user, from_host) <= 0) {
+		return (NULL);
 	}
 
 	if (strchr(sender, '\n') != NULL) {
@@ -331,8 +358,8 @@ retry:
 
 	switch (error) {
 	case 0:
-		delqueue(it);
 		syslog(LOG_INFO, "<%s> delivery successful", it->addr);
+		delqueue(it);
 		exit(EX_OK);
 
 	case 1:
@@ -422,9 +449,10 @@ main(int argc, char **argv)
 {
 	struct sigaction act;
 	char *sender = NULL;
+	char *own_name = NULL;
 	struct queue queue;
 	int i, ch;
-	int nodot = 0, showq = 0, queue_only = 0;
+	int nodot = 0, showq = 0, queue_only = 0, newaliases = 0;
 	int recp_from_header = 0;
 
 	set_username();
@@ -458,19 +486,17 @@ main(int argc, char **argv)
 	bzero(&queue, sizeof(queue));
 	LIST_INIT(&queue.queue);
 
-	if (strcmp(basename(argv[0]), "mailq") == 0) {
+	own_name = basename(argv[0]);
+
+	if (strcmp(own_name, "mailq") == 0) {
 		argv++; argc--;
 		showq = 1;
 		if (argc != 0)
 			errx(EX_USAGE, "invalid arguments");
 		goto skipopts;
-	} else if (strcmp(argv[0], "newaliases") == 0) {
-		logident_base = "dma";
-		setlogident(NULL);
-
-		if (read_aliases() != 0)
-			errx(EX_SOFTWARE, "could not parse aliases file `%s'", config.aliases);
-		exit(EX_OK);
+	} else if (strcmp(own_name, "newaliases") == 0) {
+		newaliases = 1;
+		goto skipopts;
 	}
 
 	opterr = 0;
@@ -481,7 +507,7 @@ main(int argc, char **argv)
 			if (optarg[0] == 'c' || optarg[0] == 'm') {
 				break;
 			}
-			/* else FALLTRHOUGH */
+			/* Else FALLTHROUGH */
 		case 'b':
 			/* -bX is being ignored, except for -bp */
 			if (optarg[0] == 'p') {
@@ -491,7 +517,7 @@ main(int argc, char **argv)
 				queue_only = 1;
 				break;
 			}
-			/* else FALLTRHOUGH */
+			/* Else FALLTHROUGH */
 		case 'D':
 			daemonize = 0;
 			break;
@@ -509,11 +535,15 @@ main(int argc, char **argv)
 
 		case 'o':
 			/* -oX is being ignored, except for -oi */
-			if (optarg[0] != 'i')
-				break;
-			/* else FALLTRHOUGH */
-		case 'O':
+			if (optarg[0] == 'i') {
+				nodot = 1;
+			}
 			break;
+
+		case 'O':
+			/* -O name=val is also a no-op */
+			break;
+
 		case 'i':
 			nodot = 1;
 			break;
@@ -545,7 +575,7 @@ main(int argc, char **argv)
 				doqueue = 1;
 				break;
 			}
-			/* FALLTHROUGH */
+			/* Else FALLTHROUGH */
 
 		default:
 			fprintf(stderr, "invalid argument: `-%c'\n", optopt);
@@ -596,6 +626,9 @@ skipopts:
 	if (read_aliases() != 0)
 		errlog(EX_SOFTWARE, "could not parse aliases file `%s'", config.aliases);
 
+	if (newaliases)
+		return(0);
+
 	if ((sender = set_from(&queue, sender)) == NULL)
 		errlog(EX_SOFTWARE, NULL);
 
