@@ -86,7 +86,7 @@ send_remote_command(int fd, const char* fmt, ...)
 	va_start(va, fmt);
 	s = vsnprintf(cmd, sizeof(cmd) - 2, fmt, va);
 	va_end(va);
-	if (s == sizeof(cmd) - 2 || s < 0) {
+	if (s < 0 || (size_t)s >= sizeof(cmd) - 2) {
 		strcpy(neterr, "Internal error: oversized command string");
 		return (-1);
 	}
@@ -95,25 +95,29 @@ send_remote_command(int fd, const char* fmt, ...)
 	strcat(cmd, "\r\n");
 	len = strlen(cmd);
 
-	if (((config.features & SECURETRANS) != 0) &&
-	    ((config.features & NOSSL) == 0)) {
-		while ((s = SSL_write(config.ssl, (const char*)cmd, len)) <= 0) {
-			s = SSL_get_error(config.ssl, s);
-			if (s != SSL_ERROR_WANT_READ &&
-			    s != SSL_ERROR_WANT_WRITE) {
-				strncpy(neterr, ssl_errstr(), sizeof(neterr));
-				return (-1);
+	pos = 0;
+	while (pos < len) {
+		if (((config.features & SECURETRANSFER) != 0) &&
+		    ((config.features & NOSSL) == 0)) {
+			if ((n = SSL_write(config.ssl, (const char*)(cmd + pos), len - pos)) <= 0) {
+				s = SSL_get_error(config.ssl, n);
+				if (s == SSL_ERROR_ZERO_RETURN ||
+				    s == SSL_ERROR_SYSCALL ||
+				    s == SSL_ERROR_SSL) {
+					strlcpy(neterr, ssl_errstr(), sizeof(neterr));
+					return (-1);
+				}
+				n = 0;
+			}
+		} else {
+			n = write(fd, cmd + pos, len - pos);
+			if (n < 0) {
+				if ((errno != EAGAIN) && (errno != EINTR))
+					return (-1);
+				n = 0;
 			}
 		}
-	}
-	else {
-		pos = 0;
-		while (pos < len) {
-			n = write(fd, cmd + pos, len - pos);
-			if (n < 0)
-				return (-1);
-			pos += n;
-		}
+		pos += n;
 	}
 
 	return (len);
@@ -148,15 +152,24 @@ read_remote(int fd, int extbufsize, char *extbuf)
 			memmove(buff, buff + pos, len - pos);
 			len -= pos;
 			pos = 0;
-			if (((config.features & SECURETRANS) != 0) &&
+			if (((config.features & SECURETRANSFER) != 0) &&
 			    (config.features & NOSSL) == 0) {
-				if ((rlen = SSL_read(config.ssl, buff + len, sizeof(buff) - len)) == -1) {
-					strncpy(neterr, ssl_errstr(), sizeof(neterr));
-					goto error;
+				if ((rlen = SSL_read(config.ssl, buff + len, sizeof(buff) - len)) <= 0) {
+					switch (SSL_get_error(config.ssl, rlen)) {
+					case SSL_ERROR_ZERO_RETURN:
+					case SSL_ERROR_SYSCALL:
+					case SSL_ERROR_SSL:
+						strlcpy(neterr, ssl_errstr(), sizeof(neterr));
+						goto error;
+					default:
+						/* in case of recoverable error, retry after short sleep */
+						usleep(10000);
+						continue;
+					}
 				}
 			} else {
 				if ((rlen = read(fd, buff + len, sizeof(buff) - len)) == -1) {
-					strncpy(neterr, strerror(errno), sizeof(neterr));
+					strlcpy(neterr, strerror(errno), sizeof(neterr));
 					goto error;
 				}
 			}
@@ -271,7 +284,7 @@ smtp_login(int fd, char *login, char* password, const struct smtp_features* feat
 	// LOGIN
 	if (features->auth.login) {
 		if ((config.features & INSECURE) != 0 ||
-		    (config.features & SECURETRANS) != 0) {
+		    (config.features & SECURETRANSFER) != 0) {
 			/* Send AUTH command according to RFC 2554 */
 			send_remote_command(fd, "AUTH LOGIN");
 			if (read_remote(fd, 0, NULL) != 3) {
@@ -347,7 +360,7 @@ static void
 close_connection(int fd)
 {
 	if (config.ssl != NULL) {
-		if (((config.features & SECURETRANS) != 0) &&
+		if (((config.features & SECURETRANSFER) != 0) &&
 		    ((config.features & NOSSL) == 0))
 			SSL_shutdown(config.ssl);
 		SSL_free(config.ssl);
@@ -497,7 +510,7 @@ deliver_to_host(struct qitem *it, struct mx_hostentry *host)
         } while (0)
 
 	/* Check first reply from remote host */
-	if ((config.features & SECURETRANS) == 0 ||
+	if ((config.features & SECURETRANSFER) == 0 ||
 	    (config.features & STARTTLS) != 0) {
 		config.features |= NOSSL;
 		READ_REMOTE_CHECK("connect", 2);
@@ -505,8 +518,8 @@ deliver_to_host(struct qitem *it, struct mx_hostentry *host)
 		config.features &= ~NOSSL;
 	}
 
-	if ((config.features & SECURETRANS) != 0) {
-		error = smtp_init_crypto(fd, config.features, &features);
+	if ((config.features & SECURETRANSFER) != 0) {
+		error = smtp_init_crypto(fd, config.features, &features, host->host);
 		if (error == 0)
 			syslog(LOG_DEBUG, "SSL initialization successful");
 		else
